#INCLUDE "dcdialog.CH"
#INCLUDE "pscript.CH"
#Pragma Library("dclipx.lib")

FUNCTION DC_PsBeginDoc( cPrinterName, cTitle, nOrientation, nCopies, nPageSize, ;
                        nDuplex, nPaperBin, cFont, nUnits, lAcrobat, lXps, ;
                        lPreview )

LOCAL nPrinter, aPrinters, i, aDir

IF !Empty(cFont)
  DC_PsSetFont( cFont )
ENDIF

IF Valtype(nCopies) == 'N'
  PsSetCopies(nCopies)
ENDIF

DEFAULT nUnits := APS_TEXT
PSSetUnit(nUnits)

DEFAULT lAcrobat := .f., ;
        lXps := .f., ;
        lPreview := .f.

IF lXps
  cPrinterName := 'Microsoft XPS Document Writer'
ENDIF

IF Valtype(cPrinterName) == 'C'
  aPrinters := PsGetPrinters()
  nPrinter := AScan(aPrinters,cPrinterName)
ENDIF

IF lXps .AND. nPrinter == 0
  DCMSGBOX '"Microsoft XPS Document Writer" is not installed!'
  RETURN 0
ENDIF

IF lAcrobat
   aDir := Directory('*_$$$.PDF')
   FOR i := 1 TO Len(aDir)
     FErase(aDir[i,1])
   NEXT
   PsSetFileName(_TempFileName(,'.PDF'))
   PsSetDevice(3)
ELSEIF lXps
   PsSetFileName(_TempFileName(,'.XPS'))
   PsSetDevice(1)
ELSEIF lPreview
   PsSetDevice(2)
ENDIF

RETURN PsBeginDoc( nPrinter, cTitle, nOrientation, nPageSize, nDuplex, nPaperBin )

* ---------

FUNCTION DC_PsEndDoc()

LOCAL cFileName := PSSetFileName()

PsEndDoc()

IF !Empty(cFileName)
  DC_SpawnUrl(cFileName)
ENDIF
PSSetFileName('')
RETURN nil

* ---------

FUNCTION DC_PsTextOut( nRow, nCol, xText, cPict, nAlign, cFont, nFG, nBG, nAngle, nPoint, nStyle )

LOCAL aColor := DC_Color2PS(nFG,nBG), ;
      aFont := DC_FontCompoundName2PS(cFont,nPoint,nStyle)

RETURN PsTextOut( nRow, nCol, xText, cPict, nAlign, aFont[1], aFont[2], ;
                  aFont[3], aColor[1], aColor[2], nAngle )

* ---------

FUNCTION DC_PsTextBox( nTop, nLeft, nBottom, nRight, cText, nAlign, cFont, nFG, nBG, ;
                       nThick, nPoint, nStyle )

LOCAL aColor := DC_Color2PS(nFG,nBG), ;
      aFont := DC_FontCompoundName2PS(cFont,nPoint,nStyle)

DEFAULT nThick := 1, nAlign := APS_LEFT

RETURN PsTextBox( nTop, nLeft, nBottom, nRight, cText, nAlign, aFont[1], aFont[2], ;
                  aFont[3], aColor[1], aColor[2], nThick )

* ---------

FUNCTION DC_PsSetFont( cFont, nFG, nBG )

LOCAL aFont := DC_FontCompoundName2Ps( cFont )
LOCAL aColor := DC_Color2PS( nFG, nBG )

RETURN PsSetFont( aFont[1], aFont[3], aFont[2], aColor[1], aColor[2] )

* ---------

FUNCTION DC_Color2PS( nFG, nBG )

LOCAL aColor[2]
LOCAL aPSColor := { ;
  APS_WHITE,;
  APS_BLACK,;
  APS_BBLUE,;
  APS_BRED,;
  APS_BMAGENTA,;
  APS_BGREEN,;
  APS_BCYAN,;
  APS_YELLOW,;
  APS_GRAY,;
  APS_BLUE,;
  APS_RED,;
  APS_MAGENTA ,;
  APS_GREEN,;
  APS_CYAN,;
  APS_BROWN,;
  APS_PALEGRAY }

IF Valtype(nFG) == 'N' .AND. nFG >= 0 .AND. nFG < 16
  aColor[1] := aPsColor[nFG+1]
ELSEIF Valtype(nFG) == 'N' .AND. nFG < 0
  aColor[1] := nFG
ELSEIF Valtype(nFG) == 'A'
  aColor[1] := RGB(nFG[1],nFG[2],nFG[3])
ENDIF

IF Valtype(nBG) == 'N' .AND. nBG >= 0 .AND. nBG < 16
  aColor[2] := aPsColor[nBG+1]
ELSEIF Valtype(nBG) == 'N' .AND. nBG < 0
  aColor[2] := nBG
ELSEIF Valtype(nBG) == 'A'
  aColor[2] := RGB(nBG[1],nBG[2],nBG[3])
ENDIF

RETURN aColor

* -----------

FUNCTION DC_FontCompoundName2PS(cFont,nPoint,nStyle)

LOCAL aFont[3], oFont

IF Empty(cFont) .AND. Empty(nPoint) .AND. Empty(nStyle)
  RETURN aFont
ENDIF

IF Empty(nPoint) .AND. Empty(nStyle)
  oFont := XbpFont():new():create()
  oFont:configure(cFont)
  aFont[1] := oFont:familyName
  aFont[2] := oFont:nominalPointSize
  aFont[3] := 0
  IF oFont:bold
    aFont[3] += APS_BOLD
  ENDIF
  IF oFont:italic
    aFont[3] += APS_ITALIC
  ENDIF
  IF oFont:underscore
    aFont[3] += APS_UNDERLINE
  ENDIF
  IF oFont:strikeout
    aFont[3] += APS_STRIKEOUT
  ENDIF
ELSE
  aFont[1] := cFont
  aFont[2] := nPoint
  aFont[3] := nStyle
ENDIF

RETURN aFont

* ----------

// This function returns a different file name with each call,

STATIC FUNCTION _TempFileName( cPath, cExt )

LOCAL nSeconds := Seconds(), i, aDir, cFileName

STATIC snCount := 0

DEFAULT cPath := ''
DO WHILE Seconds() - nSeconds < 100
  IF ++ snCount == 100
    snCount := 0
  ENDIF
  cFileName   := PadL( snCount, 2, "_" )    + ;
                 StrTran( Time(), ":", "" ) + '_$$$' + cExt
  IF !File(cPath+cFileName)
    EXIT
  ENDIF
ENDDO

RETURN cPath + cFileName
